"""
function for reading blockettes for ascii encodding
"""

from struct import unpack


class ReadBlocketteError(Exception):
    def __init__(self, msg):
        self.msg = msg


NOT_READ_BLOCKETTES = {1000, 1001}       # blockettes with not important info


# ________________ based on mseedpeek.libtrace.Mseed.blkXXX________________
# https://docs.python.org/3/library/struct.html
# SEED Manual V2.4 - Chapter 8 - Data Record


def read_next_blkt(b_no, data_bytes, byte_order):
    """
    :param b_no: int - next blockette Byte Number
    :param data_bytes: bytes object - file's data in byte
    :param byte_order: str - big/little endian
    :return:
        next_b_no: int - byte number of next blockette
        info: str - info read from this blockette
    """
    blockette_type = unpack('%s%s' % (byte_order, 'H'),
                            data_bytes[b_no:b_no + 2])[0]
    next_b_no = unpack('%s%s' % (byte_order, 'H'),
                       data_bytes[b_no + 2:b_no + 4])[0]
    info = ""
    try:
        if blockette_type not in NOT_READ_BLOCKETTES:
            # readBlkt will skip first 4 bytes (HH) as they are already read
            info = eval("read_blkt_%s(%s, %s, '%s')"
                        % (blockette_type, b_no, data_bytes, byte_order))
    except NameError:
        raise ReadBlocketteError(f"Function to read blockette {blockette_type}"
                                 f" isn't implemented yet.")
    return next_b_no, info


def read_blkt_500(b_no, data_bytes, byte_order):
    """
    read blockette 500 (skip first 4 bytes (HH) as they are already read)
    :param b_no: int - next blockette Byte Number
    :param data_bytes: bytes object - file's data in byte
    :param byte_order: str - big/little endian
    :return log_text: str - info of blockette 500 in string
    """
    log_text = "\nVCO Correction: %s" % unpack(
        '%s%s' % (byte_order, 'f'), data_bytes[b_no + 4:b_no + 8])[0]
    t = {}
    (t['year'], t['doy'], t['hour'], t['min'], t['sec'],
     junk, t['micro']) = unpack(
        '%s%s' % (byte_order, 'HHBBBBH'), data_bytes[b_no + 8:b_no + 18])
    log_text += ("\nTime of exception: %(year)s:%(doy)s:%(hour)s:%(min)s:"
                 "%(sec)s:%(micro)s" % t)
    log_text += "\nMicro sec: %s" % unpack(
        '%s%s' % (byte_order, 'B'), data_bytes[b_no + 18:b_no + 19])[0]
    log_text += "\nReception Quality: %s" % unpack(
        '%s%s' % (byte_order, 'B'), data_bytes[b_no + 19:b_no + 20])[0]
    log_text += "\nException Count: %s" % unpack(
        '%s%s' % (byte_order, 'I'), data_bytes[b_no + 20:b_no + 24])[0]
    log_text += "\nException Type: %s" % unpack(
        '%s%s' % (byte_order, '16s'), data_bytes[b_no + 24:b_no + 40])[
        0].strip()
    log_text += "\nClock Model: %s" % unpack(
        '%s%s' % (byte_order, '32s'), data_bytes[b_no + 40:b_no + 72])[
        0].strip()
    log_text += "\nClock Status: %s" % unpack(
        '%s%s' % (byte_order, '128s'), data_bytes[b_no + 72:b_no + 200])[
        0].strip()
    return log_text


def read_blkt_2000(b_no, data_bytes, byte_order):
    """
    read blockette 2000 (skip first 4 bytes (HH) as they are already read)
    :param b_no: int - next blockette Byte Number
    :param data_bytes: bytes object - file's data in byte
    :param byte_order: str - big/little endian
    :return log_text: str - info of blockette 2000 in string
    """

    blkt_len = unpack(
        '%s%s' % (byte_order, 'H'), data_bytes[b_no + 4:b_no + 6])[0]
    log_text = "\nTotal Blockette length: %s bytes" % blkt_len
    log_text += "\nOffset to Opaque Data: %s" % unpack(
        '%s%s' % (byte_order, 'H'), data_bytes[b_no + 6:b_no + 8])[0]
    log_text += "\nRecord number: %s" % unpack(
        '%s%s' % (byte_order, 'I'), data_bytes[b_no + 8:b_no + 12])[0]
    log_text += "\nData Word order: %s" % unpack(
        '%s%s' % (byte_order, 'B'), data_bytes[b_no + 12:b_no + 13])[0]
    log_text += "\nOpaque Data flags: %s" % unpack(
        '%s%s' % (byte_order, 'B'), data_bytes[b_no + 13:b_no + 14])[0]
    opaqueHeaderTotal = unpack(
        '%s%s' % (byte_order, 'B'), data_bytes[b_no + 14:b_no + 15])[0]
    log_text += "\nNumber of Opaque Header fields: %s" % opaqueHeaderTotal
    n = b_no + 15
    c = 0
    header_len = 0
    for i in range(opaqueHeaderTotal):
        hfield = ''
        hchar = ''
        while hchar != '~':
            hfield += hchar
            hchar = unpack(
                '%s%s' % (byte_order, '1s'), data_bytes[n + c:n + c + 1])[
                0].decode()
            header_len = c + 1
            c += 1
        log_text += "\nOpaque Header %s: %s" % (i, hfield)

    opaque_data_length = blkt_len - 15 - header_len
    log_text += "\nOpaque Data: %s" % unpack(
        '%s%s' % (byte_order, '%ss' % opaque_data_length),
        data_bytes[n + header_len:n + header_len + opaque_data_length])

    return log_text
